import { JsAstEditor, SvelteAstEditor, HtmlAstEditor, CssAstEditor } from '@svelte-add/ast-manipulation';
import { Colors } from 'picocolors/types.js';

type RemoteControlOptions = {
    workingDirectory: string;
    isTesting: boolean;
    adderOptions: Record<string, Record<string, unknown>>;
};
declare function enable(): void;
declare function isRemoteControlled(): boolean;
declare function disable(): void;

type remoteControl_RemoteControlOptions = RemoteControlOptions;
declare const remoteControl_disable: typeof disable;
declare const remoteControl_enable: typeof enable;
declare const remoteControl_isRemoteControlled: typeof isRemoteControlled;
declare namespace remoteControl {
  export { type remoteControl_RemoteControlOptions as RemoteControlOptions, remoteControl_disable as disable, remoteControl_enable as enable, remoteControl_isRemoteControlled as isRemoteControlled };
}

type ProjectType = "svelte" | "kit";

type Primitive = Readonly<string | boolean | number>;
type PromptOption<Value> = Value extends Primitive ? {
    value: Value;
    label?: string;
    hint?: string;
} : {
    value: Value;
    label: string;
    hint?: string;
};
declare function startPrompts(message: string): void;
declare function endPrompts(message: string): void;
declare function booleanPrompt(question: string, initialValue: boolean): Promise<boolean>;
declare function selectPrompt<T>(question: string, initialValue: T, options: PromptOption<T>[]): Promise<T extends infer T_1 ? T_1 extends T ? T_1 extends symbol ? never : T_1 : never : never>;
declare function textPrompt(question: string, placeholder?: string, initialValue?: string): Promise<string>;
declare function multiSelectPrompt<T>(question: string, options: PromptOption<T>[]): Promise<T[]>;
declare function groupedMultiSelectPrompt<T>(question: string, options: Record<string, PromptOption<T>[]>): Promise<T[]>;
declare function messagePrompt(title: string, content: string): void;

type prompts_PromptOption<Value> = PromptOption<Value>;
declare const prompts_booleanPrompt: typeof booleanPrompt;
declare const prompts_endPrompts: typeof endPrompts;
declare const prompts_groupedMultiSelectPrompt: typeof groupedMultiSelectPrompt;
declare const prompts_messagePrompt: typeof messagePrompt;
declare const prompts_multiSelectPrompt: typeof multiSelectPrompt;
declare const prompts_selectPrompt: typeof selectPrompt;
declare const prompts_startPrompts: typeof startPrompts;
declare const prompts_textPrompt: typeof textPrompt;
declare namespace prompts {
  export { type prompts_PromptOption as PromptOption, prompts_booleanPrompt as booleanPrompt, prompts_endPrompts as endPrompts, prompts_groupedMultiSelectPrompt as groupedMultiSelectPrompt, prompts_messagePrompt as messagePrompt, prompts_multiSelectPrompt as multiSelectPrompt, prompts_selectPrompt as selectPrompt, prompts_startPrompts as startPrompts, prompts_textPrompt as textPrompt };
}

type BooleanQuestion = {
    type: "boolean";
    default: boolean;
};
type StringQuestion = {
    type: "string";
    default: string;
};
type NumberQuestion = {
    type: "number";
    default: number;
};
type SelectQuestion<Value = any> = {
    type: "select";
    default: Value;
    options: PromptOption<Value>[];
};
type BaseQuestion = {
    question: string;
    condition?: (options: OptionValues<any>) => boolean;
};
type Question = BaseQuestion & (BooleanQuestion | StringQuestion | NumberQuestion | SelectQuestion);
type OptionDefinition = Record<string, Question>;
type OptionValues<Args extends OptionDefinition> = {
    [K in keyof Args]: Args[K] extends StringQuestion ? string : Args[K] extends BooleanQuestion ? boolean : Args[K] extends NumberQuestion ? number : Args[K] extends SelectQuestion<infer Value> ? Value : never;
};
type AvailableCliOptionKeys = keyof AvailableCliOptionKeyTypes;
type AvailableCliOptionKeyTypes = {
    default: boolean;
    path: string;
    skipPreconditions: boolean;
    skipInstall: boolean;
};
type AvailableCliOption = {
    cliArg: string;
    processedCliArg: string;
    description: string;
    allowShorthand: boolean;
} & (BooleanQuestion | StringQuestion);
type AvailableCliOptions = Record<AvailableCliOptionKeys, AvailableCliOption>;
declare const availableCliOptions: AvailableCliOptions;

type PrettierData = {
    installed: boolean;
};
type TypescriptData = {
    installed: boolean;
};
type SvelteKitData = {
    installed: boolean;
    libDirectory: string;
    routesDirectory: string;
};
type Workspace<Args extends OptionDefinition> = {
    options: OptionValues<Args>;
    cwd: string;
    prettier: PrettierData;
    typescript: TypescriptData;
    kit: SvelteKitData;
};
type WorkspaceWithoutExplicitArgs = Workspace<Record<string, Question>>;
declare function createEmptyWorkspace<Args extends OptionDefinition>(): Workspace<Args>;
declare function populateWorkspaceDetails(workspace: WorkspaceWithoutExplicitArgs, workingDirectory: string): Promise<void>;

type BaseFile<Args extends OptionDefinition> = {
    name: (options: Workspace<Args>) => string;
    condition?: ConditionDefinition<Args>;
};
type ScriptFileEditorArgs<Args extends OptionDefinition> = JsAstEditor & Workspace<Args>;
type ScriptFileType<Args extends OptionDefinition> = {
    contentType: "script";
    content: (editor: ScriptFileEditorArgs<Args>) => void;
};
type ScriptFile<Args extends OptionDefinition> = ScriptFileType<Args> & BaseFile<Args>;
type TextFileEditorArgs<Args extends OptionDefinition> = {
    content: string;
} & Workspace<Args>;
type TextFileType<Args extends OptionDefinition> = {
    contentType: "text";
    content: (editor: TextFileEditorArgs<Args>) => string;
};
type TextFile<Args extends OptionDefinition> = TextFileType<Args> & BaseFile<Args>;
type SvelteFileEditorArgs<Args extends OptionDefinition> = SvelteAstEditor & Workspace<Args>;
type SvelteFileType<Args extends OptionDefinition> = {
    contentType: "svelte";
    content: (editor: SvelteFileEditorArgs<Args>) => void;
};
type SvelteFile<Args extends OptionDefinition> = SvelteFileType<Args> & BaseFile<Args>;
type JsonFileEditorArgs<Args extends OptionDefinition> = {
    data: any;
} & Workspace<Args>;
type JsonFileType<Args extends OptionDefinition> = {
    contentType: "json";
    content: (editor: JsonFileEditorArgs<Args>) => void;
};
type JsonFile<Args extends OptionDefinition> = JsonFileType<Args> & BaseFile<Args>;
type HtmlFileEditorArgs<Args extends OptionDefinition> = HtmlAstEditor & Workspace<Args>;
type HtmlFileType<Args extends OptionDefinition> = {
    contentType: "html";
    content: (editor: HtmlFileEditorArgs<Args>) => void;
};
type HtmlFile<Args extends OptionDefinition> = HtmlFileType<Args> & BaseFile<Args>;
type CssFileEditorArgs<Args extends OptionDefinition> = CssAstEditor & Workspace<Args>;
type CssFileType<Args extends OptionDefinition> = {
    contentType: "css";
    content: (editor: CssFileEditorArgs<Args>) => void;
};
type CssFile<Args extends OptionDefinition> = CssFileType<Args> & BaseFile<Args>;
type FileTypes<Args extends OptionDefinition> = ScriptFile<Args> | TextFile<Args> | SvelteFile<Args> | JsonFile<Args> | HtmlFile<Args> | CssFile<Args>;
/**
 * @param files
 * @param workspace
 * @returns a list of paths of changed or created files
 */
declare function createOrUpdateFiles<Args extends OptionDefinition>(files: FileTypes<Args>[], workspace: Workspace<Args>): Promise<string[]>;

type PreconditionParameters<Args extends OptionDefinition> = {
    workspace: Workspace<Args>;
    fileExists: (path: string) => Promise<void>;
    fileContains: (path: string, expectedContent: string) => Promise<void>;
};
type Postcondition<Args extends OptionDefinition> = {
    name: string;
    run: (params: PreconditionParameters<Args>) => Promise<void>;
};

type ConditionDefinition<Args extends OptionDefinition> = (Workspace: Workspace<Args>) => boolean;
type WebsiteMetadata = {
    logo: string;
    keywords: string[];
    documentation: string;
};
type AdderConfigEnvironments = {
    svelte: boolean;
    kit: boolean;
};
type AdderConfigMetadata = {
    id: string;
    package: string;
    version: string;
    name: string;
    description: string;
    environments: AdderConfigEnvironments;
    website?: WebsiteMetadata;
};
type PackageDefinition<Args extends OptionDefinition> = {
    name: string;
    version: string;
    dev: boolean;
    condition?: ConditionDefinition<Args>;
};
type BaseAdderConfig<Args extends OptionDefinition> = {
    metadata: AdderConfigMetadata;
    options: Args;
    integrationType: string;
};
type InlineAdderConfig<Args extends OptionDefinition> = BaseAdderConfig<Args> & {
    integrationType: "inline";
    packages: PackageDefinition<Args>[];
    files: FileTypes<Args>[];
    nextSteps?: (data: {
        options: OptionValues<Args>;
        cwd: string;
        colors: Colors;
        docs: string | undefined;
    }) => string[];
    installHook?: (workspace: Workspace<Args>) => Promise<void>;
    uninstallHook?: (workspace: Workspace<Args>) => Promise<void>;
};
type ExternalAdderConfig<Args extends OptionDefinition> = BaseAdderConfig<Args> & {
    integrationType: "external";
    command: string;
    environment?: Record<string, string>;
};
type AdderConfig<Args extends OptionDefinition> = InlineAdderConfig<Args> | ExternalAdderConfig<Args>;
type MaybePromise<T> = Promise<T> | T;
type Precondition = {
    name: string;
    run: () => MaybePromise<{
        success: boolean;
        message: string | undefined;
    }>;
};
type AdderCheckConfig<Args extends OptionDefinition> = {
    options: Args;
    preconditions?: Precondition[];
    postconditions?: Postcondition<Args>[];
};

type AdderDetails<Args extends OptionDefinition> = {
    config: AdderConfig<Args>;
    checks: AdderCheckConfig<Args>;
};
type ExecutingAdderInfo = {
    name: string;
    version: string;
};
type AddersToApplySelectorParams = {
    projectType: ProjectType;
    addersMetadata: AdderConfigMetadata[];
};
type AddersToApplySelector = (params: AddersToApplySelectorParams) => Promise<string[]>;
declare function executeAdder<Args extends OptionDefinition>(adderDetails: AdderDetails<Args>, remoteControlOptions?: RemoteControlOptions | undefined): Promise<void>;
declare function executeAdders<Args extends OptionDefinition>(adderDetails: AdderDetails<Args>[], executingAdder: ExecutingAdderInfo, remoteControlOptions?: RemoteControlOptions | undefined, selectAddersToApply?: AddersToApplySelector | undefined): Promise<void>;
declare function determineWorkingDirectory(directory: string | undefined): string;

/**
 * @param workingDirectory
 * @returns the install status of dependencies
 */
declare function suggestInstallingDependencies(workingDirectory: string): Promise<"installed" | "skipped">;

export { type AddersToApplySelectorParams, type AvailableCliOptions, availableCliOptions, createEmptyWorkspace, createOrUpdateFiles, determineWorkingDirectory, executeAdder, executeAdders, populateWorkspaceDetails, prompts, remoteControl, suggestInstallingDependencies };
